# Dieses Programm stellt zwei Funktionen zur Verfgung, die einen +String+ in der Sprache der arithmetischen Ausdrcke
# in seine Bestandteile zerlegen bzw. ihren Tokenklassen zuordnen. Diese Funktionen werden an folgendem Beispiel getestet:
#
# 	r20 := r2 : r2:2
#
# Author:: Markus Marx, Sebastian Betz

# Diese Dateien werden fr die Unit-Tests bentigt.
require 'runit/testcase'
require 'runit/cui/testrunner'
require 'runit/testsuite'

#Definition der einzelnen Tokens als regulre Audrcke in Klassenvariablen
  @@bez = Regexp.new('\A[a-z][a-z0-9]*')
	@@reg = Regexp.new('\Ar[0-9]')
	@@zahl = Regexp.new('\Ar2:[0-9]+|[0-9]+') 
  @@op = Regexp.new('\A:=|:')
	@@tokenArray=[@@op,@@zahl,@@reg,@@bez]
	
# ==zerlege(arittxt)
# Diese Methode zerlegt einen Eingabestring in seine Bestandteile, die durch Ruby-Patterns gegeben sind
# Allerdings werden die einzelnen Teile nicht den Tokenklassen zugeordnet, sondern nur in ein Array ge
# schrieben, welches von der Methode dann zurckgegeben wird. Die Patterns sind:
#
# * bez 				= [a-z][a-z0-9]
# * reg 			  = r[0-9]
# * zahl 				= r2:[0-9]+|[0-9]+
# * op    			= :=|: 

def zerlege(arittxt)
  ergarray=Array.new
  while (arittxt.size > 0) do 
	  compop=arittxt.scan(@@op)
		compzahl=arittxt.scan(@@zahl)
		compreg=arittxt.scan(@@reg)
	  compbez=arittxt.scan(@@bez)
		comparray=[compop,compzahl,compreg,compbez]
		tokentemp=""
		comparray.each do |c|
			if (c[0]!=nil and arittxt.index(c[0])==0)
			  if (tokentemp.size<c[0].size)
			    tokentemp=c[0]
			  end
			end
		end
		ergarray.push(tokentemp)
		if(tokentemp=="")
			return Array.new
		end
		arittxt=arittxt.sub(tokentemp, '')
	end	
	return ergarray
end

# ==bestimme(arittxt)
# Die Methode "bestimme" scant einen String von Anfang bis Ende und ordnet sie den Tokenklassen zu.
# Diese Klassen sind in einem Array und einem Hash-Set als regulre Ausdrcke gespeichert. Die Ordnung 
# des Token-Array ergibt gleichzeitig auch die Prioritt der Tokens. Hier also in absteigender Reihenfolge: 
# +op+, +zahl+, +reg+, +bez+ (d.h. +Bezeichner+ haben die niedrigste Prioritt). 
#
# Das Hash-Set wird nur verwendet, weil der Programmtext leichter verstndlich ist, als bei einem mehr
# dimensionalen Array.
#
# Die Zuordnung zu den einzelnen Tokens: Dazu wird die Eingabe zunchst zerlegt. Danach werden die Tokens in der 
# entsprechend durch Prioroitten festgelegten Reihenfolge auf die Bestandteile angewendet. Sobald ein Treffer 
# gefunden ist, wird dieser mit Name und Token ausgegeben.
def bestimme(arittxt)	
	tokens=Hash.new
  while (arittxt.size > 0) do
    # Arrays fr den jeweiligen Regulren Ausdruck 
	  compop=arittxt.scan(@@op)
		compzahl=arittxt.scan(@@zahl)
		compregister=arittxt.scan(@@reg)
	  compbez=arittxt.scan(@@bez)
		comparray=[compop,compzahl,compregister,compbez]
    # Befllen des HashSet zur Identifikation der Tokens	
	  tokens[compop]="Op"
	  tokens[compzahl]="Zahl"
	  tokens[compbez]="Bez"
		tokens[compregister]="Reg"
		tokentemp=""
		tokenname=""
		# durchlaufen des comparray und Testen der Tokens der Arrays nach R1 und R2
		comparray.each do |c|
			if (c[0]!=nil and arittxt.index(c[0])==0)
			    if (tokentemp.size<c[0].size)
			      tokentemp=c[0]
				  	tokenname=tokens[c]
					end
			end
		end
		print(tokenname," - <<",tokentemp,">>  \n")
		arittxt=arittxt.sub(tokentemp, '')
	end
end

example="r20:=r2:r2:2"
#example="2:2r:2r=:02r"
#example="::=20r:r2:20:=:"

if zerlege(example).empty?
  print "Irregulaere Eingabe"
  goahead=false
else
  print("+------------------------------------------------+\n")
  print("| Zerlegung ohne Zuordnung zu den Token-Klassen: |\n")
  print("+------------------------------------------------+\n\n")
  zerlege(example).each do |comp|
	  print("<<",comp,">>  ")
  end
	print("\n\n")
	print("+-----------------------------------------------+\n")
	print("| Zerlegung mit Zuordnung zu den Token-Klassen: |\n")
	print("+-----------------------------------------------+\n\n")
	print("Akkumulierte Tokens: \n--------------------\n\n")
	bestimme(example)
end

# Testklasse
class Testing_class < RUNIT::TestCase
  # == test_zerlege
	# Der folgende Arith-Ausdruck soll korrekt zerlegt werden:
	# "r20:=r2:r2:2" 
 def test_zerlege
	string = "r20:=r2:r2:2"
	comp = ["r20",":=","r2",":","r2:2"]
	assert_equal(comp,zerlege(string))
  end
end
